from dataclasses import Field
from turtle import pd
from wencaipy.common.fetch_base_wencai import fetch_data_from_wencai
from wencaipy.common.wcParameter import QUERY_TYPE
from wencaipy.common.dataToExcel import data_to_excel


def fetch_ths_sector_index():
    """ 同花顺板块指数 :764 同花顺行业指数 同花顺特色指数  同花顺概念指数 同花顺地域指数"""
    Fields_Query = ["同花顺板块指数"]
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query,query_type=QUERY_TYPE().index)
    data_to_excel(df, "ths_all_index")
    return df

def fetch_ths_industry_level3_index():
    """ 同花顺板块指数 :764 同花顺行业指数 同花顺特色指数  同花顺概念指数 同花顺地域指数"""
    Fields_Query = ["同花顺板块指数","三级行业"]
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query,query_type=QUERY_TYPE().index)
    data_to_excel(df, "ths_level3_index")
    return df

def fetch_ths_area_index():
    """ 同花顺板块指数 :764 同花顺行业指数 同花顺特色指数  同花顺概念指数 同花顺地域指数"""
    Fields_Query = ["同花顺板块指数","同花顺地域指数"]
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query,query_type=QUERY_TYPE().index)
    data_to_excel(df, "ths_area_index")
    return df

def fetch_ths_concept_index():
    """ 同花顺板块指数 :  同花顺概念指数 """
    Fields_Query = ["同花顺板块指数","同花顺概念指数"]
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query,query_type=QUERY_TYPE().index)
    data_to_excel(df, "ths_concept_index")
    return df


def index_ths_level3_rt():
    """ 同花顺板块指数 :764 同花顺行业指数 同花顺特色指数  同花顺概念指数 同花顺地域指数"""
    Fields_Query_index = ["同花顺行业指数","三级行业"]
    Fields_return_query =  ["5日涨幅","10日涨幅","20日涨幅","60日涨幅","120日涨幅","250日涨幅"]
    Fields_Query = Fields_Query_index + Fields_return_query
    print(Fields_Query)
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query,query_type=QUERY_TYPE().index)
    print(df.columns)
    data_to_excel(df, "ths_level3_index_rt")
    return df


def index_industry_sw_ths_all():
    Fields_Query = ["申万行业指数"]
    Fields_swl = ["股票简称", "股票代码", "市盈率(pe)",  "所属申万行业",'所属同花顺行业']
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query, fields_out=Fields_swl ,query_type=QUERY_TYPE().stock)
    #print(df.columns)
    df.columns = ['sec_name','sec_code','pe','swl','thsl','trade_date']
    data_to_excel(df, "index_industry_all")
    return df


def swl_level():
    Fields_Query = ["申万行业指数"]
    Fields_swl = ["股票简称", "股票代码", "市盈率(pe)",  "所属申万行业",'所属同花顺行业']
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query, fields_out=Fields_swl ,query_type=QUERY_TYPE().stock)
    #print(df.columns)
    df.columns = ['sec_name','sec_code','pe','swl','thsl','trade_date']
    df = df[['swl']]
    df = df.drop_duplicates()
    return df 


def swl_index_components(level="L3"): # TODO
    """['股票代码', '股票简称', '所属申万行业', '新股上市日期', '最新dde大单净额', '所属概念', '总股本',
       '市盈率(pe)', '最新价', '最新涨跌幅', 'trade_date']
    """
    Fields_Query = ["所属申万行业"]
    Fields_inustry = ["股票代码","股票简称", '所属申万行业']
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query, fields_out=None, query_type=QUERY_TYPE().stock)
    #print(df.columns)
    df = df[Fields_inustry]
    df.columns = ['sec_code','sec_name','swl']
    #print(df)
    df["swl_name_L3"] = df['swl'].apply(lambda x : list(str(x).split("--"))[-1] + f"_{level}")
    data_to_excel(df, "index_swl_index_components")
    swl_L3 = list(set(df['swl_name_L3'].values))
    swl_l3_dict = dict()
    for l in swl_L3:
        swl_l3_dict.keys = l
        swl_l3_dict.values = df[df["swl_name_L3"] == l].sec_name.values
    print(swl_l3_dict)
    return df 


def thsl_index_components():
    """['股票代码', '股票简称', '所属同花顺行业', '新股上市日期', '最新dde大单净额', '所属概念', '总股本',
       '市盈率(pe)', '最新价', '最新涨跌幅', 'trade_date']
    """
    Fields_Query = ["同花顺行业"]
    Fields_inustry = ["股票简称", "股票代码", '所属同花顺行业']
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query, fields_out=Fields_inustry, query_type=QUERY_TYPE().stock)
    print(df.columns)
    df.columns = ['sec_name','sec_code','thsl','trade_date']
    for level in ["L1","L2","L3"]:
        l = int(level[-1]) - 1
        col = str(f"thsl_name_{level}")
        #print(level, l, col)
        df[col] = df['thsl'].apply(lambda x : x.split("-")[l] + f"_{level}")
    data_to_excel(df, "index_thsl_index_components")
    return df 


def index_thsl_valuation(level="L3"):
    """_summary_
    ['指数代码', '指数简称', '指数@同花顺行业指数', '指数@所属同花顺行业级别', '指数@市盈率', '指数@阶段最新市盈率日',
       '指数@阶段最新市盈率', '指数@市净率', '指数@市现率', '指数@市销率', '指数@收盘价:不复权', '指数@涨跌幅:前复权',
       'hqCode', 'marketId', 'trade_date']
    Args:
        level (str, optional): _description_. Defaults to "L3".
    Returns:
    ['指数代码', '指数简称',  ' 指数@所属同花顺行业级别', '指数@市盈率', '指数@阶段最新市盈率日',
       '指数@阶段最新市盈率', '指数@市净率', '指数@市现率', '指数@市销率', '指数@收盘价:不复权', '指数@涨跌幅:前复权',
        'trade_date']
        _type_: _description_
    """
    if level == "L2":
        Fields_Query = ["同花顺行业指数","二级行业","pe"]
    elif level == "L3":
        Fields_Query = ["同花顺行业指数","三级行业","pe"]
    Fields_index_out  =   ['指数代码', '指数简称',  '指数@所属同花顺行业级别', '指数@市盈率', '指数@阶段最新市盈率日',
       '指数@阶段最新市盈率', '指数@市净率', '指数@市现率', '指数@市销率', '指数@收盘价:不复权', '指数@涨跌幅:前复权']
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query, fields_out=Fields_index_out ,query_type=QUERY_TYPE().index)
    #print(df.columns)
    df.columns = ['index_code','index_name','level_cn','pe','pe_date','pe_new','pb','pc','ps','close','pct_chg','trade_date']
    df_out = df.copy()
    df_out['level'] = level
    df_out= df_out[['index_code','index_name','level','pe','pb','pc','ps','close','pct_chg','trade_date']]
    data_to_excel(df_out, f"index_thsl_valuation_{level}")
    return df_out 


if __name__ == "__main__":
    
    #print(fetch_ths_sector_index())
    #print(fetch_ths_industry_level3_index())
    #print(fetch_ths_area_index())
    #print(fetch_ths_concept_index())
    #print(index_ths_level3_rt())
    #print(index_industry_sw_ths_all())
    #print(swl_level())
    #print(thsl_index_components())
    #print(index_thsl_valuation("L3"))
    print(swl_index_components())


from attr import field
from wencaipy.common.fetch_base_wencai import fetch_data_from_wencai
from wencaipy.common.dataToExcel import data_to_excel
from wencaipy.common.wcParameter import QUERY_TYPE


def fetch_stock_new_high(trade_date=None, new_high_period=0):
    """   股价创历史新高 new high; history=0 / 250 """
    if new_high_period == 0:
        Fields_new_high_query = ["股价创历史新高", "所属申万行业","所属同花顺行业"] 
    else:
        Fields_new_high_query = [f"{trade_date}日股票最高价创{new_high_period}日新高"]
    print(Fields_new_high_query)
    Fields_new_high = ["股票简称", "股票代码", '最新价', '最新涨跌幅', "所属申万行业","所属同花顺行业"]
    df = fetch_data_from_wencai(trade_date=trade_date,fields_query=Fields_new_high_query, fields_out=Fields_new_high, query_type=QUERY_TYPE.stock)
    data_to_excel(df, 'stock_new_high')
    return df


def fetch_stock_hot_sort():
    """ 个股热度排名"""
    df =  fetch_data_from_wencai(fields_query=["个股热度排名"], query_type=QUERY_TYPE.stock ).sort_values(by="个股热度排名")
    data_to_excel(df,'stock_hot')
    return df

def stock_super_strong():
    """ atr值/14日均价*现价；20天上涨天数；15日均线向上；5日均线在25日上， 10日均线在25日上;业绩涨幅大于50%或者业绩预告净利润大于50%；
    """
    Fields_super_strong = ['atr值/14日均价','15日均线向上',"概念",'同花顺三级行业'] #,"最新季度收入增幅大于50%","最新季度净利润增幅大于50%"]
    #["atr值/14日均价*现价","20天上涨天数",
       #,"15日均线向上 和 5日均线在25日上 和 10日均线在25日上",
                          #"业绩涨幅大于50%或者"
                         
    df = fetch_data_from_wencai(fields_query=Fields_super_strong, query_type=QUERY_TYPE.stock)
    data_to_excel(df,'stock_super')
    return df


def WCAlpha(object):
    """  """
    def __init_(self):
        self.get_datd_wencai = fetch_data_from_wencai()
        
    def ATR(self):
        pass
    
    def BOLL(self):
        pass
    
    def UPPCT():
        pass
    
        

if __name__ == "__main__":
    
    #print(fetch_stock_new_high())
    #print(fetch_stock_hot_sort()) 
    print(stock_super_strong())   
    



"""
这里定义的是一些常用常量
"""
from enum import Enum
from pytz import timezone

CHINA_TZ = timezone("Asia/Shanghai")

startDate = "2020-04-28"  #rq_util_get_pre_trade_date(rq_util_get_last_tradedate(), 255)

MAX_QUERY_SIZE: int = 5000
TS_DATE_FORMATE: str = '%Y%m%d'
MAX_QUERY_TIMES: int = 500

SQLITE_PATH_RR_PC = "/mnt/f/Stock/iwencai/"
SQLITE_PATH_ALIYUN = "/mnt/disk1/sqlit_data"


class AdjustType(Enum):
    """
    split-adjusted type for :class:`~.zvt.contract.schema.TradableEntity` quotes
    """
    #: not adjusted
    #: 不复权
    bfq = "bfq"
    #: pre adjusted
    #: 不复权
    qfq = "qfq"
    #: post adjusted
    #: 不复权
    hfq = "hfq"


class ActorType(Enum):
    #: 个人
    individual = "individual"
    #: 公募基金
    raised_fund = "raised_fund"
    #: 社保
    social_security = "social_security"
    #: 保险
    insurance = "insurance"
    #: 外资
    qfii = "qfii"
    #: 信托
    trust = "trust"
    #: 券商
    broker = "broker"
    #: 私募
    private_equity = "private_equity"
    #: 公司(可能包括私募)
    corporation = "corporation"


class Exchange(Enum):
    #: 上证交易所
    sh = "sh"
    #: 深证交易所
    sz = "sz"
    #: bj
    bj = "bj"



class TRADE_TIMES():
    TRADE_TIME_AM = ['09:30:00','11:30:00']
    TRADE_TIME_PM = ["13:00:00","15:00:00"]
    TRADE_TIME_AM_pre = ['09:25:00','11:31:00']
    TRADE_TIME_PM_pre = ["13:00:00","15:01:00"]

#print(TRADE_TIMES().TRADE_TIME_AM)

class REPORT_PERIODS():
    q1 = '0331'
    q2 = '0630'
    q3 = '0930'
    q4 = '1231'
    REPORT_PERIODS_DICT = { 'q1': '0331',
            'q2': '0630',
            'q3': '0930',
            'q4': '1231'}
    
class PERIODS():
    """stock trade period
    """
    PERIODS=[5,10,20,60,120,250]

class SWL_LEVEL():
    LEVEL =['L1','L2','L3']
    SWL_LEVEL = ['sw_l1','sw_l2','sw_l3']



class EXCHANGE_ID():
    SSE = 'sse'  # 上交所
    SZSE = 'szse'  # 深交所
    BSE = 'bse' # 北京交易所
    SHFE = 'SHFE'  # 上期所
    DCE = 'DCE'  # 大商所
    CZCE = 'CZCE'  # 郑商所
    CFFEX = 'CFFEX'  # 中金所
    INE = 'INE'  # 能源中心
    HUOBI = 'huobi' # 火币Pro
    BINANCE = 'binance' # 币安
    BITMEX = 'bitmex' # BITMEX
    BITFINEX = 'BITFINEX' # BITFINEX
    OKEX = 'OKEx' # OKEx



class MARKET_TYPE():
    """市场种类
    日线 尾数01
    分钟线 尾数02
    tick 尾数03
    市场:
    股票 0
    指数/基金 1
    期货 2
    港股 3
    美股 4
    比特币/加密货币市场 5
    """
    STOCK_CN = 'stock_cn'  # 中国A股
    STOCK_CN_B = 'stock_cn_b'  # 中国B股
    STOCK_CN_D = 'stock_cn_d'  # 中国D股 沪伦通
    STOCK_HK = 'stock_hk'  # 港股
    STOCK_US = 'stock_us'  # 美股
    FUTURE_CN = 'future_cn'  # 国内期货
    OPTION_CN = 'option_cn'  # 国内期权
    STOCKOPTION_CN = 'stockoption_cn'  # 个股期权
    # BITCOIN = 'bitcoin'  # 比特币
    CRYPTOCURRENCY = 'cryptocurrency'  # 加密货币(衍生货币)
    INDEX_CN = 'index_cn'  # 中国指数
    FUND_CN = 'fund_cn'   # 中国基金
    BOND_CN = 'bond_cn'  # 中国债券



class CURRENCY_TYPE():
    """货币种类"""
    RMB = 'rmb'  # 人民币
    USD = 'usd'  # 美元
    EUR = 'eur'  # 欧元
    HKD = 'hkd'  # 港币
    GBP = 'GBP'  # 英镑
    BTC = 'btc'  # 比特币
    JPY = 'jpy'  # 日元
    AUD = 'aud'  # 澳元
    CAD = 'cad'  # 加拿大元


class DATASOURCE():
    """数据来源
    """

    WIND = 'wind'  # wind金融终端
    TDB = 'tdb'  # wind tdb
    THS = 'ths'  # 同花顺网页
    TUSHARE = 'tushare'  # tushare
    TDX = 'tdx'  # 通达信
    MONGO = 'mongo'  # 本地/远程Mongodb
    PGSQL = 'postgresql' # 本地/远程Mongodb
    EASTMONEY = 'eastmoney'  # 东方财富网
    CHOICE = 'choice'  # choice金融终端
    CCXT = 'ccxt'  # github/ccxt 虚拟货币
    LOCALFILE = 'localfile'  # 本地文件
    AUTO = 'auto'  # 优先从Mongodb中读取数据，不足的数据从tdx下载


class OUTPUT_FORMAT():
    """输出格式
    """

    DATASTRUCT = 'datastruct'
    DATAFRAME = 'dataframe'
    SERIES = 'series'
    NDARRAY = 'ndarray'
    LIST = 'list'
    JSON = 'json'


class RUNNING_STATUS():
    """运行状态
    starting 是一个占用状态
    100 - 202 - 200 - 400 - 500
    """

    PENDING = 100
    SUCCESS = 200
    STARTING = 202
    RUNNING = 300
    WRONG = 400
    STOPED = 500
    DROPED = 600

class FREQUENCE():
    """查询的级别
    YEAR = 'year'  # 年bar
    QUARTER = 'quarter'  # 季度bar
    MONTH = 'month'  # 月bar
    WEEK = 'week'  # 周bar
    DAY = 'day'  # 日bar
    ONE_MIN = '1min'  # 1min bar
    FIVE_MIN = '5min'  # 5min bar
    FIFTEEN_MIN = '15min'  # 15min bar
    THIRTY_MIN = '30min'  # 30min bar
    HOUR = '60min'  # 60min bar
    SIXTY_MIN = '60min'  # 60min bar
    TICK = 'tick'  # transaction
    ASKBID = 'askbid'  # 上下五档/一档
    REALTIME_MIN = 'realtime_min' # 实时分钟线
    LATEST = 'latest'  # 当前bar/latest
    2019/08/06 @yutiansut
    """

    YEAR = 'year'  # 年bar
    QUARTER = 'quarter'  # 季度bar
    MONTH = 'month'  # 月bar
    WEEK = 'week'  # 周bar
    DAY = 'day'  # 日bar
    ONE_MIN = '1min'  # 1min bar
    FIVE_MIN = '5min'  # 5min bar
    FIFTEEN_MIN = '15min'  # 15min bar
    THIRTY_MIN = '30min'  # 30min bar
    HOUR = '60min'  # 60min bar
    SIXTY_MIN = '60min'  # 60min bar
    TICK = 'tick'  # transaction
    ASKBID = 'askbid'  # 上下五档/一档
    REALTIME_MIN = 'realtime_min'  # 实时分钟线
    LATEST = 'latest'  # 当前bar/latest

DATABASE_TABLE = {
    (MARKET_TYPE.STOCK_CN, FREQUENCE.DAY): 'stock_day',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.ONE_MIN): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.FIVE_MIN): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.FIFTEEN_MIN): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.THIRTY_MIN): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.SIXTY_MIN): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.HOUR): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.TICK): 'stock_transaction',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.DAY): 'index_day',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.ONE_MIN): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.FIVE_MIN): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.FIFTEEN_MIN): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.THIRTY_MIN): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.SIXTY_MIN): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.HOUR): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.TICK): 'index_transaction',
    (MARKET_TYPE.FUND_CN, FREQUENCE.DAY): 'index_day',
    (MARKET_TYPE.FUND_CN, FREQUENCE.ONE_MIN): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.FIVE_MIN): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.FIFTEEN_MIN): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.THIRTY_MIN): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.SIXTY_MIN): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.HOUR): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.TICK): 'index_transaction',
    
}


class IntervalLevel(Enum):
    """
    Repeated fixed time interval, e.g, 5m, 1d.
    """

    #: level tick
    LEVEL_TICK = "tick"
    #: 1 minute
    LEVEL_1MIN = "1m"
    #: 5 minutes
    LEVEL_5MIN = "5m"
    #: 15 minutes
    LEVEL_15MIN = "15m"
    #: 30 minutes
    LEVEL_30MIN = "30m"
    #: 1 hour
    LEVEL_1HOUR = "1h"
    #: 4 hours
    LEVEL_4HOUR = "4h"
    #: 1 day
    LEVEL_1DAY = "1d"
    #: 1 week
    LEVEL_1WEEK = "1wk"
    #: 1 month
    LEVEL_1MON = "1mon"

    def to_pd_freq(self):
        if self == IntervalLevel.LEVEL_1MIN:
            return "1min"
        if self == IntervalLevel.LEVEL_5MIN:
            return "5min"
        if self == IntervalLevel.LEVEL_15MIN:
            return "15min"
        if self == IntervalLevel.LEVEL_30MIN:
            return "30min"
        if self == IntervalLevel.LEVEL_1HOUR:
            return "1H"
        if self == IntervalLevel.LEVEL_4HOUR:
            return "4H"
        if self >= IntervalLevel.LEVEL_1DAY:
            return "1D"

    def floor_timestamp(self, pd_timestamp):
        if self == IntervalLevel.LEVEL_1MIN:
            return pd_timestamp.floor("1min")
        if self == IntervalLevel.LEVEL_5MIN:
            return pd_timestamp.floor("5min")
        if self == IntervalLevel.LEVEL_15MIN:
            return pd_timestamp.floor("15min")
        if self == IntervalLevel.LEVEL_30MIN:
            return pd_timestamp.floor("30min")
        if self == IntervalLevel.LEVEL_1HOUR:
            return pd_timestamp.floor("1h")
        if self == IntervalLevel.LEVEL_4HOUR:
            return pd_timestamp.floor("4h")
        if self == IntervalLevel.LEVEL_1DAY:
            return pd_timestamp.floor("1d")

    def to_minute(self):
        return int(self.to_second() / 60)

    def to_second(self):
        return int(self.to_ms() / 1000)

    def to_ms(self):
        """
        To seconds count in the interval
        :return: seconds count in the interval
        """
        #: we treat tick intervals is 5s, you could change it
        if self == IntervalLevel.LEVEL_TICK:
            return 5 * 1000
        if self == IntervalLevel.LEVEL_1MIN:
            return 60 * 1000
        if self == IntervalLevel.LEVEL_5MIN:
            return 5 * 60 * 1000
        if self == IntervalLevel.LEVEL_15MIN:
            return 15 * 60 * 1000
        if self == IntervalLevel.LEVEL_30MIN:
            return 30 * 60 * 1000
        if self == IntervalLevel.LEVEL_1HOUR:
            return 60 * 60 * 1000
        if self == IntervalLevel.LEVEL_4HOUR:
            return 4 * 60 * 60 * 1000
        if self == IntervalLevel.LEVEL_1DAY:
            return 24 * 60 * 60 * 1000
        if self == IntervalLevel.LEVEL_1WEEK:
            return 7 * 24 * 60 * 60 * 1000
        if self == IntervalLevel.LEVEL_1MON:
            return 31 * 7 * 24 * 60 * 60 * 1000

    def __ge__(self, other):
        if self.__class__ is other.__class__:
            return self.to_ms() >= other.to_ms()
        return NotImplemented

    def __gt__(self, other):

        if self.__class__ is other.__class__:
            return self.to_ms() > other.to_ms()
        return NotImplemented

    def __le__(self, other):
        if self.__class__ is other.__class__:
            return self.to_ms() <= other.to_ms()
        return NotImplemented

    def __lt__(self, other):
        if self.__class__ is other.__class__:
            return self.to_ms() < other.to_ms()
        return NotImplemented


class BlockCategory(Enum):
    #: 行业版块
    industry = "industry"
    #: 概念版块
    concept = "concept"
    #: 区域版块
    area = "area"


class IndexCategory(Enum):
    scope = "scope"
    #: 行业指数
    industry = "industry"
    #: 风格指数
    style = "style"
    #: fund
    fund = "fund"
   


class ReportPeriod(Enum):
    # 有些基金的2，4季报只有10大持仓，半年报和年报有详细持仓，需要区别对待
    season1 = "season1"
    season2 = "season2"
    season3 = "season3"
    season4 = "season4"
    half_year = "half_year"
    year = "year"



import os
import datetime
from pprint import pprint
import requests
import pandas as pd
import numpy as np
import pickle
from wencaipy.common.dataToExcel import data_to_excel, read_data_from_excel

rest_days_df = None
chinese_holidays_range = None


def today():
    return datetime.date.today()


def date2str(cursor_date):
    """
    explanation:
        对输入日期进行格式化处理，返回格式为 "%Y-%m-%d" 格式字符串
        支持格式包括:
        1. str: "%Y%m%d" "%Y%m%d%H%M%S", "%Y%m%d %H:%M:%S",
                "%Y-%m-%d", "%Y-%m-%d %H:%M:%S", "%Y-%m-%d %H%M%S"
        2. datetime.datetime
        3. pd.Timestamp
        4. int -> 自动在右边加 0 然后转换，譬如 '20190302093' --> "2019-03-02"
    params:
        * cursor_date->
            含义: 输入日期
            类型: str
            参数支持: []
    """
    if isinstance(cursor_date, datetime.datetime):
        cursor_date = str(cursor_date)[:10]
    elif isinstance(cursor_date, np.datetime64):
        cursor_date = str(cursor_date)[:10]
    elif isinstance(cursor_date, str):
        try:
            cursor_date = str(pd.Timestamp(cursor_date))[:10]
        except:
            raise ValueError('请输入正确的日期格式, 建议 "%Y-%m-%d"')
    elif isinstance(cursor_date, int):
        cursor_date = str(pd.Timestamp("{:<014d}".format(cursor_date)))[:10]
    else:
        raise ValueError('请输入正确的日期格式，建议 "%Y-%m-%d"')
    return cursor_date


def _init_chinese_rest_days(headers=None):
    global rest_days_df, chinese_holidays_range
    if rest_days_df is None:
        url = os.getenv("TQ_CHINESE_HOLIDAY_URL", "https://files.shinnytech.com/shinny_chinese_holiday.json")
        rsp = requests.get(url, timeout=30, headers=headers)
        chinese_holidays = rsp.json()
        _first_day = datetime.date(int(chinese_holidays[0].split('-')[0]), 1, 1)  # 首个日期所在年份的第一天
        _last_day = datetime.date(int(chinese_holidays[-1].split('-')[0]), 12, 31)  # 截止日期所在年份的最后一天
        chinese_holidays_range = (_first_day, _last_day)
        rest_days_df = pd.DataFrame(data={'date': pd.Series(pd.to_datetime(chinese_holidays, format='%Y-%m-%d'))})
        rest_days_df['trading_restdays'] = False  # 节假日为 False
    return chinese_holidays_range


def get_trading_calendar(start_dt=None, end_dt=None, headers=None):
    """   
    """
    _init_chinese_rest_days(headers=headers)
    df = pd.DataFrame()
    df['date'] = pd.Series(pd.date_range(start=start_dt, end=end_dt, freq="D"))
    df['trading'] = df['date'].dt.dayofweek.lt(5)
    result = pd.merge(rest_days_df, df, sort=True, how="right", on="date")
    result.fillna(True, inplace=True)
    df['trading'] = result['trading'] & result['trading_restdays']
    #df.to_pickle("trade_date_sse.pkl")
    data_to_excel(df, "trade_date_sse")
    return df


def trade_date_sse(start_dt='1990-01-01', end_dt='2030-12-31'):
    if not end_dt:
        end_dt = today()
    #print(type(end_dt))
    try:
        df = read_data_from_excel("trade_date_sse")
        #print(df)
    except:
        df = get_trading_calendar(start_dt, end_dt)
        
    td = df[df.trading == True]["date"].values
    td = list(map(lambda x: date2str(x), td))
    return td

trade_date_cn = trade_date_sse()

if __name__ == "__main__":
    #print(read_data_from_excel("trade_date_sse"))
    print(trade_date_cn)




# -*- coding:utf-8 -*-
import logging
import re
import requests
import pandas as pd
from wencaipy.core.cons import WENCAI_CRAWLER_URL, WENCAI_HEADERS
from wencaipy.core.content import BackTest, YieldBackTest, EventBackTest, LastJs
from wencaipy.core.cookies import WencaiCookie
from wencaipy.core.session import Session
from wencaipy.common.wcParameter import (MAX_QUERY_SIZE, QUERY_TYPE,Question_Url,headers_wc,
                                         Drop_columns_ipo, Drop_columns_ipo_2, Fields_basic, Fields_query_etf, Fields_query_index)
from wencaipy.utils.tradeDate import today, if_traded_now, get_real_trade_date, get_last_tradedate

from wencaipy.common.wcParameter import Fields_query_condbond
from wencaipy.common.tradeDate import get_real_trade_date, if_traded_now, get_last_tradedate

class Wencai(object):
    logging.basicConfig(
        level=logging.WARNING,
        format='%(asctime)s [%(levelname)s] %(message)s',
    )

    def __init__(self, cn_col=False, proxies=None, verify=False):
        self.cookies = WencaiCookie()
        self.cn_col = cn_col
        self.session = Session(proxies=proxies, verify=verify)

    def backtest(self, query, start_date, end_date, period, benchmark):
        payload = {
            "query": query,
            "start_date": start_date,
            "end_date": end_date,
            "period": period,
            "benchmark": benchmark
        }

        r = self.session.post_result(source='backtest', url=WENCAI_CRAWLER_URL['backtest'], data=payload)
        if r.status_code == 200:
            print(r.json())
            return BackTest(content=r.json(), cn_col=self.cn_col, start_date=start_date, end_date=end_date,
                            session=self.session)

        else:
            raise Exception(r.content.decode('utf-8'))

    def yieldbacktest(self, query, start_date, end_date, stock_hold, upper_income, lower_income, period, fall_income,
                      day_buy_stock_num):
        payload = {
            "query": query,
            "start_date": start_date,
            "end_date": end_date,
            "period": period,
            "stock_hold": stock_hold,
            "upper_income": upper_income,
            "lower_income": lower_income,
            "fall_income": fall_income,
            "day_buy_stock_num": day_buy_stock_num
        }
        r = self.session.post_result(WENCAI_CRAWLER_URL['yieldbacktest'], data=payload,
                                     add_headers=WENCAI_HEADERS['backtest'], source='backtest')
        if r.status_code == 200:
            return YieldBackTest(content=r.json(), cn_col=self.cn_col, query=query,
                                 start_date=start_date, end_date=end_date,
                                 session=self.session)
        else:
            raise Exception(r.content.decode('utf-8'))

    def eventbacktest(self, query, index_code, period, start_date, end_date):
        payload = {
            "query": query,
            "start_date": start_date,
            "end_date": end_date,
            "period": period,
            "index_code": index_code
        }

        r = self.session.post_result(WENCAI_CRAWLER_URL['eventbacktest'], data=payload, source='eventbacktest')
        if r.status_code == 200:
            return EventBackTest(content=r.json(), cn_col=self.cn_col)
        else:
            raise Exception(r.content.decode('utf-8'))

    def lastjs(self, code):
        r = self.session.get_result(WENCAI_CRAWLER_URL['lastjs'].format(code), source='lastjs',
                                    add_headers=WENCAI_HEADERS['lastjs'])
        if r.status_code == 200:
            return LastJs(content=r.text, code=code).get_data
        else:
            raise Exception(r.content.decode('utf-8'))

    def search(self, query_string): #TODO

        payload = {
            "question": query_string,
            "page": 1,
            "perpage": 100,
            "log_info": '{"input_type": "typewrite"}',
            "source": "Ths_iwencai_Xuangu",
            "version": 2.0,
            "secondary_intent": "",
            "query_area": "",
            "block_list": "",
            "add_info": '{"urp": {"scene": 1, "company": 1, "business": 1}, "contentType": "json", "searchInfo": true}'
        }

        r = self.session.post_result(url=WENCAI_CRAWLER_URL['search'],
                                     data=payload, force_cookies=True)
        result = r.json()['data']['answer'][0]['txt'][0]['content']['components'][0]['data']['datas']

        def _re_str(x: str):
            _re = re.findall('(.*):前复权', x)
            if len(_re) >= 1:
                x = _re[-1]
            check_date = re.search(r"(\d{4}\d{1,2}\d{1,2})",x)
            if check_date is not None:
                return x.replace('[{}]'.format(check_date.group()), '')
            else:
                return x

        data = pd.DataFrame().from_dict(result)
        if not data.empty:
            columns = {i: _re_str(i) for i in data.columns}
            data = data.rename(columns=columns)
            for col in ['market_code', 'code', '关键词资讯', '涨跌幅']:
                if col in data.columns:
                    del data[col]
        return data
    
    
    def search_data(self, trade_date=None, fields_query=None):
        if not trade_date:
            if if_traded_now():
                trade_date = today()
            else:
                trade_date= get_last_tradedate()
        trade_date = get_real_trade_date(trade_date)
        print(trade_date)
        
        payload = {
            # 查询问句
            "question": "{},{},上市日期<={}".format(trade_date, ",".join(fields_query), trade_date),
            # 返回查询记录总数 
            "perpage": MAX_QUERY_SIZE,
            "query_type": QUERY_TYPE.stock
        }
        try:
            response = requests.get(Question_Url, params=payload, headers=headers_wc)
            if response.status_code == 200:
                json = response.json()
                df_data = pd.DataFrame(json["data"]["data"])
                print(df_data.columns)
                #print(df_data)
                # 规范返回的columns，去掉[xxxx]内容
                df_data.columns = [col.split("[")[0] for col in df_data.columns]
                #print(df_data)
                # 筛选查询字段，非查询字段丢弃
                df = df_data 
              
                df = df.assign(trade_date = trade_date) #, code=df["股票代码"].apply(lambda x: x[0:6])).set_index("trade_date", drop=True)
                return df
            else:
                print(f"连接访问接口失败")           
        except Exception as e:
            print(e)

        



#!/usr/bin/env python
#  -*- coding: utf-8 -*-
"""
交易日历处理
"""
import os
import datetime
from typing import Union, List
import numpy
import pandas as pd
import requests
from sqlalchemy import false
from wencaipy.common.wcParameter import  TRADE_TIMES
from wencaipy.common.trade_calander import trade_date_sse

trade_date_cn = trade_date_sse()

def today():
    return datetime.date.today()


def date2str(cursor_date):
    """
    explanation:
        对输入日期进行格式化处理，返回格式为 "%Y-%m-%d" 格式字符串
        支持格式包括:
        1. str: "%Y%m%d" "%Y%m%d%H%M%S", "%Y%m%d %H:%M:%S",
                "%Y-%m-%d", "%Y-%m-%d %H:%M:%S", "%Y-%m-%d %H%M%S"
        2. datetime.datetime
        3. pd.Timestamp
        4. int -> 自动在右边加 0 然后转换，譬如 '20190302093' --> "2019-03-02"
    params:
        * cursor_date->
            含义: 输入日期
            类型: str
            参数支持: []
    """
    if isinstance(cursor_date, datetime.datetime):
        cursor_date = str(cursor_date)[:10]
    elif isinstance(cursor_date, numpy.datetime64):
        cursor_date = str(cursor_date)[:10]
    elif isinstance(cursor_date, str):
        try:
            cursor_date = str(pd.Timestamp(cursor_date))[:10]
        except:
            raise ValueError('请输入正确的日期格式, 建议 "%Y-%m-%d"')
    elif isinstance(cursor_date, int):
        cursor_date = str(pd.Timestamp("{:<014d}".format(cursor_date)))[:10]
    else:
        raise ValueError('请输入正确的日期格式，建议 "%Y-%m-%d"')
    return cursor_date


def get_last_tradedate():
    _now =  datetime.datetime.now()
    _hour = _now.hour
    _date = _now.strftime('%Y-%m-%d')
    #print(_now, _hour, _date)
    if  _date in trade_date_cn:
        if _hour > 17:
            return trade_date_cn[trade_date_cn.index(_date)]
        else:
            return trade_date_cn[trade_date_cn.index(_date) - 1]
    while _date not in trade_date_cn:
        _date = pd.to_datetime(str(_date)) + pd.Timedelta(days=-1)
        _date = _date.strftime('%Y-%m-%d')
    #print(_date) 
    #print(trade_date_sse[trade_date_sse.index(_date)])
    return trade_date_cn[trade_date_sse.index(_date)]


def if_trade_date(date):
    if str(date) in trade_date_sse():
        return True
    else:
        return False


def if_traded_now():
    """
    '日期是否交易 and time > 9:30'
    """
    _now = datetime.datetime.now()
    _now_time =  str(datetime.datetime.strptime(str(_now)[0:19],  "%Y-%m-%d %H:%M:%S"))[11:]
    if (today() in trade_date_cn) and (str(_now_time) >  TRADE_TIMES().TRADE_TIME_AM[0]):
        return True
    else:
        return False


def if_trade_datetime(_time=datetime.datetime.now()):
    """  时间是否交易 """
    _time = datetime.datetime.strptime(str(_time)[0:19], "%Y-%m-%d %H:%M:%S")
    if if_trade_date(str(_time.date())[0:10]):
        if _time.hour in [10, 13, 14]:
            return True
        elif (
           _time.hour in [9] and _time.minute >= 15
            ):
            return True
        elif _time.hour in [11] and _time.minute <= 30:
            return True
        else:
                return False
    else:
        return False


def get_real_trade_date(date=str(today()), trade_list=trade_date_cn, towards=-1):
    """
    explanation:
        获取真实的交易日期
    params:
        * date->
            含义: 日期
            类型: date
            参数支持: []
        * trade_list->
            含义: 交易列表
            类型: List
            参数支持: []
        * towards->
            含义: 方向， 1 -> 向前, -1 -> 向后
            类型: int
            参数支持: [1， -1]
    """
    date = str(date)[0:10]
    if towards == 1:
        if pd.Timestamp(date) >= pd.Timestamp(trade_list[-1]):
            return trade_list[-1]
        while date not in trade_list:
            date = str(
                datetime.datetime.strptime(str(date)[0:10], "%Y-%m-%d")
                + datetime.timedelta(days=1)
            )[0:10]
        else:
            return str(date)[0:10]
    elif towards == -1:
        if pd.Timestamp(date) <= pd.Timestamp(trade_list[0]):
            return trade_list[0]
        while date not in trade_list:
            date = str(
                datetime.datetime.strptime(str(date)[0:10], "%Y-%m-%d")
                - datetime.timedelta(days=1)
            )[0:10]
        else:
            return str(date)[0:10]


if __name__ == "__main__":
    print(today())
    #end_dt = str(datetime.now().date())
    #df = get_trading_calendar("2022-01-01",end_dt)
    #print(df)
    #tradedate = trade_date_sse(start_dt='2022-07-01', end_dt=end_dt)
    #print(type(tradedate))
    #print(tradedate)
    #print(trade_date_sse()[:5])
    #print(datetime.datetime.now())
    #print(datetime.time(9,25,0))
    #print(type(trade_date_cn[-1]))
    #print(trade_date_cn[-1])
 
    print(if_traded_now())
    print(if_trade_date("2022-07-08"))
    print(if_trade_datetime())
    
    print(get_real_trade_date("2022-07-08"))
    
    
    
    
   
    
    
    
 
   


import re
import pandas as pd

from dataToExcel import read_data_from_excel
from wencaipy.common.tradeDate import get_real_trade_date


df = read_data_from_excel(f"stock_prs_oh_volchg_inc_forecast_concept_{get_real_trade_date()}")
df.to_html("stock_prs.html",header=True, index=False)


import time
import datetime
from typing import List, Tuple, Union
import numpy as np
import pandas as pd

from wencaipy.utils.rqParameter import FREQUENCE, MARKET_TYPE
from wencaipy.utils.tradeDate import trade_date_cn

from wencaipy.utils.tradeDate import trade_date_sse

trade_date_sse = trade_date_cn

def rq_util_format_date2str(cursor_date):
    """
    explanation:
        对输入日期进行格式化处理，返回格式为 "%Y-%m-%d" 格式字符串
        支持格式包括:
        1. str: "%Y%m%d" "%Y%m%d%H%M%S", "%Y%m%d %H:%M:%S",
                "%Y-%m-%d", "%Y-%m-%d %H:%M:%S", "%Y-%m-%d %H%M%S"
        2. datetime.datetime
        3. pd.Timestamp
        4. int -> 自动在右边加 0 然后转换，譬如 '20190302093' --> "2019-03-02"
    params:
        * cursor_date->
            含义: 输入日期
            类型: str
            参数支持: []
    """
    if isinstance(cursor_date, datetime.datetime):
        cursor_date = str(cursor_date)[:10]
    elif isinstance(cursor_date, np.datetime64):
        cursor_date = str(cursor_date)[:10]
    elif isinstance(cursor_date, str):
        try:
            cursor_date = str(pd.Timestamp(cursor_date))[:10]
        except:
            raise ValueError('请输入正确的日期格式, 建议 "%Y-%m-%d"')
    elif isinstance(cursor_date, int):
        cursor_date = str(pd.Timestamp("{:<014d}".format(cursor_date)))[:10]
    else:
        raise ValueError('请输入正确的日期格式，建议 "%Y-%m-%d"')
    return cursor_date


def rq_util_get_next_period(datetime, frequence="1min"):
    """
    得到给定频率的下一个周期起始时间
    :param datetime: 类型 datetime eg: 2018-11-11 13:01:01
    :param frequence: 类型 str eg: '30min'
    :return: datetime eg: 2018-11-11 13:31:00
    """
    freq = {
        FREQUENCE.YEAR: "Y",
        FREQUENCE.QUARTER: "Q",
        FREQUENCE.MONTH: "M",
        FREQUENCE.WEEK: "W",
        FREQUENCE.DAY: "D",
        FREQUENCE.SIXTY_MIN: "60T",
        FREQUENCE.THIRTY_MIN: "30T",
        FREQUENCE.FIFTEEN_MIN: "15T",
        FREQUENCE.FIVE_MIN: "5T",
        FREQUENCE.ONE_MIN: "T",
    }
    return (pd.Period(datetime, freq=freq[frequence]) + 1).to_timestamp()


def rq_util_get_next_trade_date(
    cursor_date: Union[str, pd.Timestamp, datetime.datetime] = None, n: int = 1
) -> str:
  
    if not cursor_date:
        cursor_date = datetime.date.today().strftime("%Y-%m-%d")
    else:
        cursor_date = pd.Timestamp(cursor_date).strftime("%Y-%m-%d")
    if cursor_date in trade_date_sse:
        # 如果指定日期为交易日
        return trade_date_sse[trade_date_sse.index(cursor_date) + n]
    real_trade_date = rq_util_get_real_date(cursor_date, towards=-1)
    return trade_date_sse[trade_date_sse.index(real_trade_date) + n]


def rq_util_get_pre_trade_date(
    cursor_date: Union[str, pd.Timestamp, datetime.datetime] = None, n: int = 1
) -> str:
    if not cursor_date:
        cursor_date = datetime.date.today().strftime("%Y-%m-%d")
    else:
        cursor_date = pd.Timestamp(cursor_date).strftime("%Y-%m-%d")
    if cursor_date in trade_date_sse:
        return trade_date_sse[trade_date_sse.index(cursor_date) - n]
    real_trade_date = rq_util_get_real_date(cursor_date, towards=1)
    return trade_date_sse[trade_date_sse.index(real_trade_date) - n]


def rq_util_get_last_tradedate():
    _now =  datetime.datetime.now()
    _hour = _now.hour
    _date = _now.strftime('%Y-%m-%d')
    #print(_now, _hour, _date)
    if  _date in trade_date_sse:
        if _hour > 17:
            return trade_date_sse[trade_date_sse.index(_date)]
        else:
            return trade_date_sse[trade_date_sse.index(_date) - 1]
    while _date not in trade_date_sse:
        _date = pd.to_datetime(str(_date)) + pd.Timedelta(days=-1)
        _date = _date.strftime('%Y-%m-%d')
    #print(_date) 
    #print(trade_date_sse[trade_date_sse.index(_date)])
    return trade_date_sse[trade_date_sse.index(_date)]


def rq_util_if_trade(day):
    """
    得到前 n 个交易日 (不包含当前交易日)
    '日期是否交易'
    查询上面的 交易日 列表
    :param day: 类型 str eg: 2018-11-11
    :return: Boolean 类型
    """
    if day in trade_date_sse:
        return True
    else:
        return False


def rq_util_if_traded_now():
    """
    '日期是否交易 and time > 9:30'
    查询上面的 交易日 列表
    :param day: 类型 str eg: 2018-11-11
    :return: Boolean 类型
    """
    _now = datetime.datetime.now()
    _now_time =  datetime.datetime.strptime(str(_now)[0:19], "%Y-%m-%d %H:%M:%S")
    print(_now_time)
    
    if (datetime.date.today() in trade_date_sse) and (datetime.datetime.now() > datetime.time(9, 25, 0)):
        return True
    else:
        return False


def rq_util_if_tradetime(
    _time=datetime.datetime.now(), market=MARKET_TYPE.STOCK_CN, code=None
):
    """
    explanation:
        时间是否交易
    params:
        * _time->
            含义: 指定时间
            类型: datetime
            参数支持: []
        * market->
            含义: 市场
            类型: int
            参数支持: [MARKET_TYPE.STOCK_CN]
        * code->
            含义: 代码
            类型: str
            参数支持: [None]
    """
    _time = datetime.datetime.strptime(str(_time)[0:19], "%Y-%m-%d %H:%M:%S")
    if market is MARKET_TYPE.STOCK_CN:
        if rq_util_if_trade(str(_time.date())[0:10]):
            if _time.hour in [10, 13, 14]:
                return True
            elif (
                _time.hour in [9] and _time.minute >= 15
            ):  # 修改成9:15 加入 9:15-9:30的盘前竞价时间
                return True
            elif _time.hour in [11] and _time.minute <= 30:
                return True
            else:
                return False
        else:
            return False
    elif market is MARKET_TYPE.FUTURE_CN:
        date_today = str(_time.date())
        date_yesterday = str((_time - datetime.timedelta(days=1)).date())

        is_today_open = rq_util_if_trade(date_today)
        is_yesterday_open = rq_util_if_trade(date_yesterday)

        # 考虑周六日的期货夜盘情况
        if is_today_open == False:  # 可能是周六或者周日
            if is_yesterday_open == False or (
                _time.hour > 2 or _time.hour == 2 and _time.minute > 30
            ):
                return False

        shortName = ""  # i , p
        for i in range(len(code)):
            ch = code[i]
            if ch.isdigit():  # ch >= 48 and ch <= 57:
                break
            shortName += code[i].upper()

        period = [[9, 0, 10, 15], [10, 30, 11, 30], [13, 30, 15, 0]]

        if shortName in ["IH", "IF", "IC"]:
            period = [[9, 30, 11, 30], [13, 0, 15, 0]]
        elif shortName in ["T", "TF"]:
            period = [[9, 15, 11, 30], [13, 0, 15, 15]]

        if 0 <= _time.weekday() <= 4:
            for i in range(len(period)):
                p = period[i]
                if (
                    _time.hour > p[0] or (_time.hour == p[0] and _time.minute >= p[1])
                ) and (
                    _time.hour < p[2] or (_time.hour == p[2] and _time.minute < p[3])
                ):
                    return True

        # 最新夜盘时间表_2019.03.29
        nperiod = [
            [["AU", "AG", "SC"], [21, 0, 2, 30]],
            [["CU", "AL", "ZN", "PB", "SN", "NI"], [21, 0, 1, 0]],
            [["RU", "RB", "HC", "BU", "FU", "SP"], [21, 0, 23, 0]],
            [
                [
                    "A",
                    "B",
                    "Y",
                    "M",
                    "JM",
                    "J",
                    "P",
                    "I",
                    "L",
                    "V",
                    "PP",
                    "EG",
                    "C",
                    "CS",
                ],
                [21, 0, 23, 0],
            ],
            [["SR", "CF", "RM", "MA", "TA", "ZC", "FG", "IO", "CY"], [21, 0, 23, 30]],
        ]

        for i in range(len(nperiod)):
            for j in range(len(nperiod[i][0])):
                if nperiod[i][0][j] == shortName:
                    p = nperiod[i][1]
                    condA = _time.hour > p[0] or (
                        _time.hour == p[0] and _time.minute >= p[1]
                    )
                    condB = _time.hour < p[2] or (
                        _time.hour == p[2] and _time.minute < p[3]
                    )
                    # in one day
                    if p[2] >= p[0]:
                        if (
                            (_time.weekday() >= 0 and _time.weekday() <= 4)
                            and condA
                            and condB
                        ):
                            return True
                    else:
                        if (
                            (_time.weekday() >= 0 and _time.weekday() <= 4) and condA
                        ) or (
                            (_time.weekday() >= 1 and _time.weekday() <= 5) and condB
                        ):
                            return True
                    return False
        return False


def rq_util_get_next_day(date, n=1):
    """
    explanation:
        得到下一个(n)交易日
    params:
        * date->
            含义: 日期
            类型: str
            参数支持: []
        * n->
            含义: 步长
            类型: int
            参数支持: [int]
    """
    date = str(date)[0:10]
    return rq_util_date_gap(date, n, "gt")


def rq_util_get_last_day(date, n=1):
    """
    explanation:
       得到上一个(n)交易日
    params:
        * date->
            含义: 日期
            类型: str
            参数支持: []
        * n->
            含义: 步长
            类型: int
            参数支持: [int]
    """
    date = str(date)[0:10]
    return rq_util_date_gap(date, n, "lt")


def rq_util_get_last_datetime(datetime, day=1):
    """
    explanation:
        获取几天前交易日的时间
    params:
        * datetime->
            含义: 指定时间
            类型: datetime
            参数支持: []
        * day->
            含义: 指定时间
            类型: int
            参数支持: []
    """

    date = str(datetime)[0:10]
    return "{} {}".format(rq_util_date_gap(date, day, "lt"), str(datetime)[11:])


def rq_util_get_next_datetime(datetime, day=1):
    date = str(datetime)[0:10]
    return "{} {}".format(rq_util_date_gap(date, day, "gt"), str(datetime)[11:])


def rq_util_get_real_date(date, trade_list=trade_date_sse, towards=-1):
    """
    explanation:
        获取真实的交易日期
    params:
        * date->
            含义: 日期
            类型: date
            参数支持: []
        * trade_list->
            含义: 交易列表
            类型: List
            参数支持: []
        * towards->
            含义: 方向， 1 -> 向前, -1 -> 向后
            类型: int
            参数支持: [1， -1]
    """
    date = str(date)[0:10]
    if towards == 1:
        if pd.Timestamp(date) >= pd.Timestamp(trade_list[-1]):
            return trade_list[-1]
        while date not in trade_list:
            date = str(
                datetime.datetime.strptime(str(date)[0:10], "%Y-%m-%d")
                + datetime.timedelta(days=1)
            )[0:10]
        else:
            return str(date)[0:10]
    elif towards == -1:
        if pd.Timestamp(date) <= pd.Timestamp(trade_list[0]):
            return trade_list[0]
        while date not in trade_list:
            date = str(
                datetime.datetime.strptime(str(date)[0:10], "%Y-%m-%d")
                - datetime.timedelta(days=1)
            )[0:10]
        else:
            return str(date)[0:10]


def rq_util_get_real_datelist(start, end):
    """
    explanation:
        取数据的真实区间，当start end中间没有交易日时返回None, None,
        同时返回的时候用 start,end=rq_util_get_real_datelist
    params:
        * start->
            含义: 开始日期
            类型: date
            参数支持: []
        * end->
            含义: 截至日期
            类型: date
            参数支持: []
    """
    real_start = rq_util_get_real_date(start, trade_date_sse, 1)
    real_end = rq_util_get_real_date(end, trade_date_sse, -1)
    if trade_date_sse.index(real_start) > trade_date_sse.index(real_end):
        return None, None
    else:
        return (real_start, real_end)


def rq_util_get_trade_range(start, end):
    """
    explanation:
       给出交易具体时间
    params:
        * start->
            含义: 开始日期
            类型: date
            参数支持: []
        * end->
            含义: 截至日期
            类型: date
            参数支持: []
    """
    start, end = rq_util_get_real_datelist(start, end)
    if start is not None:
        return trade_date_sse[
            trade_date_sse.index(start) : trade_date_sse.index(end) + 1 : 1
        ]
    else:
        return None


def rq_util_get_trade_gap(start, end):
    """
    explanation:
        返回start_day到end_day中间有多少个交易天 算首尾
    params:
        * start->
            含义: 开始日期
            类型: date
            参数支持: []
        * end->
            含义: 截至日期
            类型: date
            参数支持: []
    """
    start, end = rq_util_get_real_datelist(start, end)
    if start is not None:
        return trade_date_sse.index(end) + 1 - trade_date_sse.index(start)
    else:
        return 0


def rq_util_date_gap(date, gap, methods):
    """
    explanation:
        返回start_day到end_day中间有多少个交易天 算首尾
    params:
        * date->
            含义: 字符串起始日
            类型: str
            参数支持: []
        * gap->
            含义: 间隔多数个交易日
            类型: int
            参数支持: [int]
        * methods->
            含义: 方向
            类型: str
            参数支持: ["gt->大于", "gte->大于等于","小于->lt", "小于等于->lte", "等于->==="]
    """
    try:
        if methods in [">", "gt"]:
            return trade_date_sse[trade_date_sse.index(date) + gap]
        elif methods in [">=", "gte"]:
            return trade_date_sse[trade_date_sse.index(date) + gap - 1]
        elif methods in ["<", "lt"]:
            return trade_date_sse[trade_date_sse.index(date) - gap]
        elif methods in ["<=", "lte"]:
            return trade_date_sse[trade_date_sse.index(date) - gap + 1]
        elif methods in ["==", "=", "eq"]:
            return date

    except:
        return "wrong date"


def rq_util_get_trade_datetime(dt=datetime.datetime.now()):
    """
    explanation:
        获取交易的真实日期
    params:
        * dt->
            含义: 时间
            类型: datetime
            参数支持: []
    """

    # dt= datetime.datetime.now()

    if rq_util_if_trade(str(dt.date())) and dt.time() < datetime.time(15, 0, 0):
        return str(dt.date())
    else:
        return rq_util_get_real_date(str(dt.date()), trade_date_sse, 1)



if __name__ == "__main__":
    print(rq_util_get_last_tradedate())
    #print(rq_util_get_pre_trade_date('2022-05-20',3))
    #print(rq_util_get_trade_range('2022-05-18', '2022-05-20'))
    print(rq_util_if_trade(datetime.date.today()))
    #print(rq_util_get_trade_datetime(dt=datetime.datetime.now()))
    #print(rq_util_get_real_date("2022-06-30"))
    print(rq_util_if_traded_now())

    
    
    


from wencaipy.common.fetch_base_wencai import fetch_data_from_wencai
from wencaipy.common.wcParameter import QUERY_TYPE
from wencaipy.common.dataToExcel import data_to_excel



def fetch_ths_all_index_prs():
    """ 同花顺板块指数 :764 同花顺行业指数 同花顺特色指数  同花顺概念指数 同花顺地域指数"""
    Fields_Query_ths_level3 = ["同花顺板块指数"]
    Fields_return_query =  ["5日涨幅", "10日涨幅","20日涨幅","60日涨幅","120日涨幅","250日涨幅"]
    Fields_Query =  Fields_Query_ths_level3 + Fields_return_query
    #print(Fields_Query)
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query,query_type=QUERY_TYPE().index)
    df.columns = ['指数代码', '指数简称', '指数@同花顺板块指数', 
                "5日涨幅", "10日涨幅","20日涨幅","60日涨幅","120日涨幅","250日涨幅",       
               '指数@收盘价:不复权', '指数@涨跌幅:前复权',
                'hqCode','marketId','trade_date']
    data_to_excel(df, "ths_all_index_rt")
    return df



def fetch_ths_level3_index_prs():
    """ 同花顺板块指数 :764 同花顺行业指数 同花顺特色指数  同花顺概念指数 同花顺地域指数"""
    Fields_Query_ths_level3 = ["同花顺板块指数","三级行业"]
    Fields_return_query =  ["5日涨幅", "10日涨幅","20日涨幅","60日涨幅","120日涨幅","250日涨幅"]
    Fields_Query =  Fields_Query_ths_level3 + Fields_return_query
    print(Fields_Query)
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query,query_type=QUERY_TYPE().index)
    df.columns = ['指数代码', '指数简称', '指数@同花顺板块指数', '指数@所属同花顺行业级别',
                "5日涨幅", "10日涨幅","20日涨幅","60日涨幅","120日涨幅","250日涨幅",       
                '指数@同花顺行业指数', '指数@收盘价:不复权', '指数@涨跌幅:前复权',
                'hqCode','marketId','trade_date']
    data_to_excel(df, "ths_level3_index_rt")
    return df


if __name__ == "__main__":
    print(fetch_ths_all_index_prs())
    print(fetch_ths_level3_index_prs())
    


""" 
new_high_history
perids_return; --> rs; oh,ol;ma

close > bull(upper)
            vol > 2*vol(50)
            close > ma(20) hold  to 1w
            oh > 85, ol > 25
            上市时间满1年，历史新高， 申万行业
            一周涨幅；一个月涨幅； 3个月涨幅；一年涨幅； 行业

fields_qeury = ['营业收入(同比增长率)','归属母公司股东的净利润(同比增长率)', '销售毛利率','净资产收益率roe',"所属申万行业"]
fields_qeury = ["一周涨幅,一个月涨幅, 3个月涨幅,六个月涨幅,一年涨幅, 申万行业, 同花顺行业"]

"""


import pandas as pd
from wencaipy.common.wcParameter import SQLITE_PATH_RR_PC 


def data_to_excel(data=None,file_name=None):
    try:
        data.to_excel(f"{SQLITE_PATH_RR_PC }/{file_name}.xlsx")
        print(f"Save data to excel <{file_name}.xlsx> finish !")
    except Exception as e:
        print(e)


def read_data_from_excel(file_name=None):
    try:
        return pd.read_excel(f"{SQLITE_PATH_RR_PC }/{file_name}.xlsx",index_col=0)
        print(f"Read data from excel <{file_name}.xlsx> finish !")
    except Exception as e:
        print(e)
        

if __name__ == "__main__":
    print(read_data_from_excel("trade_date_sse"))
    print(read_data_from_excel("stock_ma"))
    


# -*- coding:utf-8 -*-


WENCAI_LOGIN_URL = {
    "scrape_transaction": 'http://www.iwencai.com/traceback/strategy/transaction',
    "scrape_report": 'http://www.iwencai.com/traceback/strategy/submit',
    'strategy': 'http://www.iwencai.com/traceback/strategy/submit',
    "search": "http://www.iwencai.com/data-robot/extract-new",
    'recommend_strategy': 'http://www.iwencai.com/traceback/list/get-strategy',
    'backtest': 'http://backtest.10jqka.com.cn/backtest/app.html#/backtest',
    'lastjs':'http://x.10jqka.com.cn/stockpick?tid=stockpick&ts=1&qs=pc_~soniu~stock~stock~znxg~topbar&allow_redirect=false&zhineng=opened'
}

WENCAI_CRAWLER_URL = {
    'history_detail': 'http://backtest.10jqka.com.cn/backtestonce/historydetail?sort_by=desc&id={backtest_id}&start_date={start_date}&end_date={end_date}&period={period}',
    "backtest": "http://backtest.10jqka.com.cn/backtestonce/backtest",
    "yieldbacktest": "http://backtest.10jqka.com.cn/tradebacktest/yieldbacktest",
    "history_pick": 'http://backtest.10jqka.com.cn/tradebacktest/historypick?query={query}&hold_num={hold_num}&trade_date={trade_date}',
    'eventbacktest': 'http://backtest.10jqka.com.cn/eventbacktest/backtest',
    'lastjs':'http://d.10jqka.com.cn/v2/time/{}/last.js',
    'search': 'http://x.10jqka.com.cn/unifiedwap/unified-wap/v2/result/get-robot-data'

}

WENCAI_HEADERS = {
    'backtest': {
        'Host': "backtest.10jqka.com.cn",
        'Origin': "http://backtest.10jqka.com.cn",
        "Referer": "http://backtest.10jqka.com.cn/backtest/app.html",
    },
    'lastjs':{
        'Host': "d.10jqka.com.cn",
        # 'Origin': "http://backtest.10jqka.com.cn",
        "Referer": "http://x.10jqka.com.cn/",
        "User-Agent":"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.93 Safari/537.36"
    }

}


# -*- coding:utf-8 -*-
from wencaipy.core.crawler import Wencai

global_cn_col = True
global_proxies = None
verify = False


def set_variable(cn_col=True, proxies=None, is_verify=False):
    global global_cn_col, global_proxies, verify
    global_cn_col = cn_col
    global_proxies = proxies
    verify = is_verify


def get_scrape_report(query, start_date, end_date, period, benchmark):
    return Wencai(cn_col=globals()['global_cn_col'], proxies=globals()['global_proxies'],
                  verify=globals()['verify']).backtest(
        query=query, start_date=start_date, end_date=end_date, period=period, benchmark=benchmark
    )


def get_strategy(query, start_date, end_date, stock_hold, upper_income, lower_income, period, fall_income,
                 day_buy_stock_num):
    return Wencai(cn_col=globals()['global_cn_col'], proxies=globals()['global_proxies'],
                  verify=globals()['verify']).yieldbacktest(query=query,
                                                            start_date=start_date,
                                                            end_date=end_date,
                                                            period=period,
                                                            fall_income=fall_income,
                                                            day_buy_stock_num=day_buy_stock_num,
                                                            upper_income=upper_income,
                                                            lower_income=lower_income,
                                                            stock_hold=stock_hold)

def get_event_evaluate(query, start_date, end_date, period, index_code):
    return Wencai(cn_col=globals()['global_cn_col'], proxies=globals()['global_proxies'],
                  verify=globals()['verify']).eventbacktest(query=query, start_date=start_date, end_date=end_date,
                                                            period=period, index_code=index_code)


def get_lastjs(code):
    return Wencai(cn_col=globals()['global_cn_col'],
                  proxies=globals()['global_proxies'], verify=globals()['verify']).lastjs(code)


def search(query):
    return Wencai(proxies=globals()['global_proxies'], verify=globals()['verify']).search(query_string=query)


def search_data(query):
    return Wencai(proxies=globals()['global_proxies'], verify=globals()['verify']).search_data(fields_query=query)





__version__ = '0.1.0'
# -*- coding:utf-8 -*-
import sys
import os
from wencaipy.core.event import (get_scrape_report,
                               set_variable,
                               get_strategy,
                               get_event_evaluate,
                               get_lastjs,
                               search,
                               search_data
                               )





# -*- coding:utf-8 -*-
import requests
import random
from wencaipy.core.cookies import WencaiCookie


class Session(requests.Session):
    headers = {
        "Accept": "application/json,text/javascript,*/*;q=0.01",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.8",
        'Connection': 'keep-alive',
        'Content-Type': "application/x-www-form-urlencoded; charset=UTF-8",
        'User-Agent': "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36",
        'X-Requested-With': "XMLHttpRequest"

    }

    def __init__(self, proxies=None, verify=False):
        requests.Session.__init__(self)
        self.headers.update(Session.headers)
        if proxies is not None:
            if not isinstance(proxies, (list, dict)):
                raise TypeError('proxies should be list or dict')
            if isinstance(proxies, list):
                proxies = random.choice(proxies)
        self.proxies = proxies
        self.verify = verify

    def update_headers(self, source, add_headers, force_cookies=False):
        if force_cookies:
            self.headers['hexin-v'] = WencaiCookie().getHeXinVByHttp()
        else:
            self.headers['hexin-v'] = WencaiCookie().getHexinVByJson(source=source)
        if add_headers is not None:
            if not isinstance(add_headers, dict):
                raise TypeError('update_headers should be `dict` type.')
            for k, v in add_headers.items():
                self.headers[k] = v

    def get_result(self, url, source=None, force_cookies=False, add_headers=None, **kwargs):
        self.update_headers(add_headers=add_headers, source=source, force_cookies=force_cookies)
        if self.proxies is None:
            return super(Session, self).get(url=url, **kwargs)
        else:
            return super(Session, self).get(url=url, proxies=self.proxies, verify=self.verify, **kwargs)

    def post_result(self, url, source=None, data=None, json=None, add_headers=None, force_cookies=False, **kwargs):
        self.update_headers(add_headers=add_headers, source=source, force_cookies=force_cookies)
        if self.proxies is None:
            return super(Session, self).post(url=url, data=data, json=json, **kwargs)
        else:
            return super(Session, self).post(url=url, data=data, json=json, proxies=self.proxies, verify=self.verify,
                                             **kwargs)


import os
import json
import execjs  #pip install PyExecJS 
import datetime as dt


class WencaiCookie:

    def __init__(self):
        self.json_path = os.path.dirname(__file__) + '/cookies.json'

    def getHeXinVByHttp(self):
        with open(os.path.dirname(os.path.dirname(__file__)) + '/js/hexin.js', 'r') as f:
            jscontent = f.read()
        context = execjs.compile(jscontent)
        return context.call("v")

    def setHexinByJson(self, source, cookies=None):
        if cookies is None: cookies = dict()
        henxin_v = self.getHeXinVByHttp()
        cookies[source] = henxin_v
        cookies['expire_time'] = dt.datetime.today().strftime('%Y-%m-%d %H:%M:%S')
        with open(self.json_path, 'w') as f:
            json.dump(cookies, f)
        return henxin_v

    def is_expire(self, expire_time, days=3):
        delta = dt.datetime.today() - dt.datetime.strptime(expire_time, '%Y-%m-%d %H:%M:%S')
        if delta.days >= days:
            return True
        else:
            return False

    def getHexinVByJson(self, source):
        json_path = os.path.dirname(__file__) + '/cookies.json'
        if os.path.exists(json_path):
            with open(json_path, 'r') as f:
                cookies = json.load(f)

            if source in cookies:
                if not self.is_expire(cookies['expire_time']):
                    return cookies[source]
                else:
                    return self.setHexinByJson(source=source, cookies=cookies)
            else:
                return self.setHexinByJson(source=source, cookies=cookies)
        else:
            self.setHexinByJson(source=source)


import re
import json
import pandas as pd
from wencaipy.core.cons import WENCAI_CRAWLER_URL

pd.set_option('display.max_rows', 2000)


class BackTest:

    def __init__(self, content, cn_col, start_date, end_date, session):
        if content['errorcode'] == 1:
            raise ValueError(content['errormsg'])
        self.content = content['result']
        self.cn_col = cn_col
        self.start_date = start_date
        self.end_date = end_date
        self.session = session

    @property
    def backtest_data(self):
        _ = {'daySaleStrategy': '持有期',
             'averageChangeRate': '平均区间涨跌幅',
             'hs300AverageIncome': '同期基准',
             'highProbability': '次日高开概率',
             'averageOpenIncome': '次日开盘平均涨跌幅',
             'maxChangeRate': '最大涨跌幅',
             'minChangeRate': '最小涨跌幅',
             'averageLossRatio': '盈亏比',
             "winRate": '上涨概率',
             "hs300WinRate": '基准上涨概率',
             'changeRateGroup': '涨跌分布图'}
        data = self.content['backtestData']
        data = pd.DataFrame().from_dict(data)
        if 'changeRateGroup' in data.columns:
            del data['changeRateGroup']
        if self.cn_col:
            data = data.rename(columns=_)
        return data

    @property
    def condition_data(self):
        return self.content['conditionData']

    @property
    def report_data(self):
        data = self.content['reportData']
        _ = {
            'historyHappenCount': '历史发生次数',
            'maxAverageChangeRate': '最优平均涨跌幅',
            'maxWinRate': '最大上涨概率'
        }
        response = {'maxAverageChangeRate': data['maxAverageChangeRate'][0],
                    'maxWinRate': data['maxWinRate'][0],
                    'historyHappenCount': data['historyHappenCount']}
        if self.cn_col:
            response = {_[k]: v for k, v in response.items()}

        return response

    def history_detail(self, period, start_date=None, end_date=None):
        if start_date is None:
            start_date = self.start_date

        if end_date is None:
            end_date = self.end_date

        url = WENCAI_CRAWLER_URL['history_detail'].format(backtest_id=self.content['id'],
                                                          start_date=start_date,
                                                          end_date=str(end_date),
                                                          period=str(period))
        context = self.session.get_result(url=url, source='history_detail').text
        context = re.findall('{"result":(.*?),"errorcode":0,"errormsg":""}', context)[0]
        data = json.loads(context)
        data = pd.DataFrame().from_dict(data)
        _ = {
            'stock_code': '股票代码',
            'stock_name': '股票简称',
            "start_date": "起始时间",
            "end_date": "终止时间",
            "start_price": "起始价",
            "end_price": "终止价",
            "change_rate": "区间涨跌幅",
            "stock_market": "股票市场"
        }
        if self.cn_col:
            data = data.rename(columns=_)
        return data


class YieldBackTest:
    def __init__(self, content, cn_col, query, start_date, end_date, session):
        self.content = content['result']
        self.cn_col = cn_col
        self.query = query
        self.start_date = start_date
        self.end_date = end_date
        self.session =session

    @property
    def profit_data(self):
        hq300_data = self.content['Hq300Data']
        profit_data = self.content['profitData'][0]
        hq300_data = [{'date': i['timestamp'], 'benchmark_value': i['value']} for i in hq300_data]
        profit_data = [{'date': i[0], 'profit_value': i[1]} for i in profit_data['everydayIncome']]
        hq300_data = pd.DataFrame().from_dict(hq300_data)
        profit_data = pd.DataFrame().from_dict(profit_data)
        profit_data = pd.merge(left=hq300_data, right=profit_data, how='left', on='date')
        profit_data = profit_data.loc[:, ['date', 'benchmark_value', 'profit_value']]
        return profit_data

    @property
    def condition_data(self):
        return self.content['conditionData']

    @property
    def backtest_data(self):
        data = self.content['backtestData'][0]
        if 'profitVolatility' in data:
            del data['profitVolatility']

        _ = {
            'annualYield': '预期年化收益率',
            'averageIncome': '单次收益平均值',
            'averageLossRatio': '盈亏比',
            'daySaleStrategy': '持有期',
            'maxDrawDown': '最大回撤',
            'maxIncome': '单次最大收益值',
            'minIncome': '单次收益最小值',
            'profitVolatility': '',
            'sharpeRatio': '夏普比率',
            'totalTradeTimes': '交易次数',
            'weekWinRate': '周战胜率',
            'winRate': '成功率'
        }
        if self.cn_col:
            data = {_[k]: v for k, v in data.items()}
        return data

    @property
    def score_data(self):
        data = self.content['scoreData']
        _ = {
            'annualYield': '绝对收益',
            'maxDrawDown': '抗风险能力',
            'profitVolatility': '稳定性',
            'score': '综合得分',
            'winRate': '选股能力',
            'averageLossRatio': '盈利能力',
            'rank': '排名'
        }
        for i in ['date', 'count']:
            if i in data:
                del data[i]

        if self.cn_col:
            data = {_[k]: v for k, v in data.items()}
        return data

    def history_pick(self, trade_date, hold_num=1):
        url = WENCAI_CRAWLER_URL['history_pick'].format(trade_date=trade_date, hold_num=hold_num, query=self.query)
        context = self.session.get_result(url, source='history_detail').text
        context = re.findall('{"result":(.*?),"errorcode":0,"errormsg":""}', context)[0]
        data = json.loads(context)['stocks']
        data = pd.DataFrame().from_dict(data)

        _ = {
            'change_rate': '涨跌幅',
            'close_price': '当日收盘价（元)',
            'dde': 'dde大单净量（%）',
            "equit_scale": "股本规模",
            "turnover_rate": "换手率",
            "stock_code": "股票代码",
            "stock_market": "股票市场",
            "stock_name": "股票名称",
        }
        if self.cn_col:
            data = data.rename(columns=_)
        return data

    def history_detail(self, period, start_date=None, end_date=None):

        if start_date is None:
            start_date = self.start_date

        if end_date is None:
            end_date = self.end_date

        url = WENCAI_CRAWLER_URL['history_detail'].format(backtest_id=self.content['id'], start_date=start_date,
                                                          end_date=end_date, period=period)
        context = self.session.get_result(url, source='history_detail').text
        context = re.findall('{"result":(.*?),"errorcode":0,"errormsg":""}', context)[0]
        data = json.loads(context)
        data = pd.DataFrame().from_dict(data)
        _ = {
            'stock_code': '股票代码',
            'stock_name': '股票简称',
            "start_date": "起始时间",
            "end_date": "终止时间",
            "start_price": "起始价",
            "end_price": "终止价",
            "change_rate": "区间涨跌幅",
            "stock_market": "股票市场"
        }
        if self.cn_col:
            data = data.rename(columns=_)
        return data


class EventBackTest:
    def __init__(self, content, cn_col):
        self.cn_col = cn_col
        self.content = content['result']

    @property
    def event_list(self):
        data = self.content['data']
        data = pd.DataFrame().from_dict(data)

        _ = {
            '1day': '次日涨跌幅',
            'date': '事件日期',
            'index_code': '涉及标的'
        }
        if self.cn_col:
            data = data.rename(columns=_)
        return data

    @property
    def report_data(self):
        data = self.content['reportData']
        _ = {
            'maxChangeRate': '最大上涨概率',
            'maxTotalCount': '历史发生次数',
            'maxWinRate': '最优平均涨跌幅'
        }

        if self.cn_col:
            data = {_[k]: v for k, v in data.items()}
        return data

class LastJs:
    def __init__(self, content, code):
        index_str = 'quotebridge_v2_time_{}_last'.format(code)
        content = re.findall( index_str + '(.*)', content)[0].replace('(','').replace(')','')
        self.content = json.loads(content)[code]

    @property
    def get_data(self):
        data = self.content['data']
        data = [i.split(',') for i in data.split(';')]
        data = pd.DataFrame().from_records(data)
        return data


import datetime
import pandas as pd
from pprint import pprint
import requests

from wencaipy.common.wcParameter import (MAX_QUERY_SIZE, QUERY_TYPE,Question_Url,headers_wc,
                                         Drop_columns_ipo, Drop_columns_ipo_2, Fields_basic, Fields_query_etf, Fields_query_index)
from wencaipy.utils.tradeDate import today, if_traded_now, get_real_trade_date, get_last_tradedate
from wencaipy.common.dataToExcel import data_to_excel
from wencaipy.common.wcParameter import Fields_query_condbond
from wencaipy.common.tradeDate import get_real_trade_date, if_traded_now, get_last_tradedate


def fetch_data_from_wencai(trade_date=None, fields_query=None,fields_out=None, query_type=None):
    """ """
    if not trade_date:
        if if_traded_now():
            trade_date = today()
        else:
            trade_date= get_last_tradedate()
    trade_date = get_real_trade_date(trade_date)
    print(trade_date)

    if not fields_query:
        if query_type==QUERY_TYPE().fund:
            fields_query = Fields_query_etf
        if query_type==QUERY_TYPE().stock:
            fields_query = Fields_basic
        if query_type==QUERY_TYPE().bond:
            fields_query=Fields_query_condbond
        if query_type==QUERY_TYPE().index:
            fields_query=Fields_query_index
            
    if not query_type:
        query_type = QUERY_TYPE().stock
        if query_type==QUERY_TYPE().fund:
            fields_query = Fields_query_etf
        if query_type==QUERY_TYPE().stock:
            fields_query = Fields_basic
        if query_type==QUERY_TYPE().bond:
            fields_query=Fields_query_condbond
        if query_type==QUERY_TYPE().index:
            fields_query=Fields_query_index

        
    payload = {
        # 查询问句
        "question": "{},{},上市日期<={}".format(trade_date, ",".join(fields_query), trade_date),
        # 返回查询记录总数 
        "perpage": MAX_QUERY_SIZE,
        "query_type": QUERY_TYPE().stock
    }
    try:
        response = requests.get(Question_Url, params=payload, headers=headers_wc)
        if response.status_code == 200:
            json = response.json()
            df_data = pd.DataFrame(json["data"]["data"])
            print(df_data.columns)
            #print(df_data)
            # 规范返回的columns，去掉[xxxx]内容
            df_data.columns = [col.split("[")[0] for col in df_data.columns]
            #print(df_data)
            # 筛选查询字段，非查询字段丢弃
            if not fields_out:
                if query_type == QUERY_TYPE().stock:
                    try:
                        df = df_data.drop(columns=Drop_columns_ipo)
                    except:
                        df = df_data.drop(columns=Drop_columns_ipo_2)    
                        #print(df)
                else:
                    df = df_data 
            else:
                df = df_data[fields_out]
            # 增加列, 交易日期 code 设置索引、
            dfc = df.copy()
            df = df.assign(trade_date = trade_date) #, code=df["股票代码"].apply(lambda x: x[0:6])).set_index("trade_date", drop=True)
            return df
        else:
            print(f"连接访问接口失败")           
    except Exception as e:
        print(e)


if __name__ == "__main__":
    fields = ["收盘价不复权","成交量", "复权因子"]
    fields = ["最高价创250日新高"]
    fields_out = ["股票代码","股票简称",	"开盘价:不复权","最高价:不复权","最低价:不复权", "收盘价:不复权","成交量", "成交额","复权因子", "新股上市日期"]

    #data_to_excel(fetch_data_from_wencai(trade_date, fields, fields_out),f"stock_daily_{trade_date}")
    #print(fetch_data_from_wencai().columns)
 
    df = fetch_data_from_wencai(trade_date=None,fields_query=["最高价创60日新高"], query_type=QUERY_TYPE.stock)
    print(df)
 
    #print(fetch_data_from_wencai(query_type=QUERY_TYPE().fund))
    #print(fetch_data_from_wencai(query_type=QUERY_TYPE().index))
    #print(fetch_data_from_wencai(query_type=QUERY_TYPE().bond))
    #print(fetch_data_from_wencai(query_type=QUERY_TYPE().stock))
   
    
    



from wencaipy.stock import stock_prs

import streamlit as st

from wencaipy.common.dataToExcel import read_data_from_excel
from wencaipy.common.tradeDate import get_real_trade_date
from wencaipy.stock.stock_prs import stock_prs_oh_ol_volchg_inc_forecast_concept

def save_stock_prs_to_excel():
    #stock_prs.stock_prs_oh_ol_volchg_inc()
    #stock_prs.stock_prs_oh_ol_volchg_inc_concept()
    stock_prs_oh_ol_volchg_inc_forecast_concept()
    
def stock_streamlit():
    df = read_data_from_excel(f'stock_prs_oh_volchg_inc_forecast_concept_{get_real_trade_date()}')
    st.write(df)
    
if __name__ == "__main__":
    #save_stock_prs_to_excel()
    stock_streamlit()


import time
import pandas as pd 
from wencaipy.common.fetch_base_wencai import fetch_data_from_wencai
from wencaipy.common.wcParameter import QUERY_TYPE, PERIODS, PERIODS_LIST
from wencaipy.common.dataToExcel import data_to_excel
from wencaipy.utils.tradeDate import get_real_trade_date


def stock_industry_concept(trade_date=None):
   Fields_Query = ["申万行业","概念"]
   df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query,query_type=QUERY_TYPE().stock)
   print(df.columns)
   """
   ['股票代码', '股票简称', '所属申万行业', '所属概念', '新股上市日期', '所属概念数量', 'a股市值(不含限售股)',
       '所属同花顺行业', '最新dde大单净额', '总股本', '市盈率(pe)', '最新价', '最新涨跌幅', 'trade_date']
   """
   df.columns = ['sec_code', 'sec_name', 'swl', 'concept', 'list_date', 'concept_num', 'fmv', \
      'thsl ', 'dde', 'total_shares',  'pe', 'close', 'pct_chg', 'trade_date']
   data_to_excel(df, "stock_industry_concept")
   return df


def stock_ma():
    Fields_ma_query =  ["5日均线", "10日均线","20日均线","60日均线","120日均线","250日均线"]
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_ma_query, query_type=QUERY_TYPE().stock)
    """
    df.columns = ['股票代码', '股票简称', '5日均线', '10日均线', '20日均线', '60日均线', '120日均线', '250日均线',
       '新股上市日期', '买入信号inter', '技术形态', '所属同花顺行业', '最新价', '最新涨跌幅', 'trade_date']
    """
    #print(df.columns)
    df.columns = ['sec_code', 'sec_name', 'ma5', 'ma10', 'ma20', 'ma60', 'ma120', 'ma250',
       'list_date', 'buy_flag', 'technical_status', 'thsl', 'close', 'pct_chg', 'trade_date']
    data_to_excel(df, "stock_ma")
    return df


def stock_oh_ol(period=250):
   """
   fields_in:  
   ['股票代码', '股票简称', '收盘价:不复权[20220718]', '历史最高价前复权',
       '{(}收盘价:不复权[20220718]{/}历史最高价前复权{)}', '区间最高价:前复权[20210707-20220718]',
       '区间最高价:前复权日[20210707-20220718]',
       '{(}收盘价:不复权[20220718]{/}区间最高价:前复权[20210707-20220718]{)}',
       '区间最低价:前复权[20210707-20220718]', '区间最低价:前复权日[20210707-20220718]',
       '{(}收盘价:不复权[20220718]{/}区间最低价:前复权[20210707-20220718]{)}', '新股上市日期',
       '开盘价:不复权[20220718]', '最高价:不复权[20220718]', '最低价:不复权[20220718]', '申购日期',
       '单账户顶格', '公开发行市值', '网上发行数量', '网下发行数量', '网上有效申购户数', '上市板块', '新股主承销商',
       '网上中签结果公告日', '招股说明书', '打新技巧', '所属同花顺行业', '最新涨跌幅', 'hqCode', 'marketId']
   field_out:   
   ['股票代码', '股票简称', '收盘价:不复权', '历史最高价前复权', '{(}收盘价:不复权', '区间最高价:前复权',
       '区间最高价:前复权日', '{(}收盘价:不复权', '区间最低价:前复权', '区间最低价:前复权日', '{(}收盘价:不复权',
       '新股上市日期', '开盘价:不复权', '最高价:不复权', '最低价:不复权', '所属同花顺行业', '最新涨跌幅',
       'trade_date']
   """ 
   Fields_high_period = ["历史最高价","250日最高价"]
   Fields_high_period_chg = ["股价/历史最高价","股价/250日最高价","股价/250日最低价"]
   Fields_high_period_out = ['股票代码', '股票简称', '收盘价:不复权', '历史最高价前复权', '{(}收盘价:不复权', '区间最高价:前复权',
       '区间最高价:前复权日', '{(}收盘价:不复权', '区间最低价:前复权', '区间最低价:前复权日', '{(}收盘价:不复权',
       '新股上市日期', '开盘价:不复权', '最高价:不复权', '最低价:不复权', '所属同花顺行业', '最新涨跌幅',
       'trade_date']
   df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_high_period_chg, query_type=QUERY_TYPE().stock)
   """
    df.columns = ['股票代码', '股票简称', '收盘价', '历史最高价', '收盘价/历史最高价', '区间最高价',
       '区间最高价日', '收盘价/区间最高价', '区间最低价', '区间最低价日', '收盘价/区间最低价日',
       '新股上市日期', '开盘价', '最高价', '最低价', '所属同花顺行业', '最新涨跌幅', 'trade_date']
   """
   #print(df)
   #print(df.columns)
   df = df[['股票代码', '股票简称', '收盘价:不复权', '历史最高价前复权', '{(}收盘价:不复权', 
            '区间最高价:前复权', '区间最高价:前复权日',  '区间最低价:前复权', '区间最低价:前复权日', 
       '新股上市日期', '开盘价:不复权', '最高价:不复权', '最低价:不复权', '所属同花顺行业', '最新涨跌幅',
       'trade_date']]

   df.columns = ['sec_code', 'sec_name', 'close', 'H_0', 'OH_0',f'OH_{period}',  f'OL_{period}',
                 f'H_{period}', f'date_H_{period}',   f'L_{period}',f'date_L_{period}',
               'list_date', 'open', 'high', 'low', 'thsl', 'pch_chg', 'trade_date']
   #print(df.columns)
   df['OH_0'] = 100 *  df['OH_0'] 
   df[f'OH_{period}']=  100 * df[f'OH_{period}']
   df[f'OL_{period}'] =   100 * (df[f'OL_{period}'] - 1)
   data_to_excel(df, f"stock_high_low_chg_{period}")
   return df


def stock_rt():
    """ """
    Fields_industry = ["所属申万行业",'所属同花顺行业']
    Fields_return_query =  ["5日涨幅", "10日涨幅","20日涨幅","60日涨幅","120日涨幅","250日涨幅"]
    Fields_Query =  Fields_industry  + Fields_return_query
    #print(Fields_Query)
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query,query_type=QUERY_TYPE().stock)
    df.columns = ['股票代码', '股票简称','所属申万行业', '所属同花顺行业',
                "5日涨幅", "10日涨幅","20日涨幅","60日涨幅","120日涨幅","250日涨幅",       
                "新股上市日期",'最新dde大单净额', '所属概念','最新价', '最新涨跌幅','trade_date']
    data_to_excel(df, "stock_rt")
    return df


def stock_vol_chg(period_vol=50):
    Fields_Query = [f"成交量/{period_vol}日成交均量"]
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query,query_type=QUERY_TYPE().stock)
    """
    df.columns = ['股票代码', '股票简称', '成交量', f'{period}日均成交量', f'成交量变化_{period}', '新股上市日期', '涨跌幅:前复权', '涨跌',
       '振幅', '成交额', '所属同花顺行业', f'{period}日成交量', f'{period}日换手率', '最新价', '最新涨跌幅', 'trade_date']
    #print(df.columns)
    """
    df.columns = ['sec_code', 'sec_name', 'vol', f'vol_ma{period_vol}', f'vol_chg{period_vol}', 'list_date', 'pct_chg', 'change',
       'amplitude ', 'amount', 'thsl', f'vol{period_vol}', f'turnover{period_vol}', 'close', 'pct_chg', 'trade_date']
    data_to_excel(df, f"stock_vol_chg_{period_vol}")
    return df


def stock_prs():
    """ """
    Fields_industry = ["所属申万行业",'所属同花顺行业','市值']
    Fields_return_query =  ["5日涨幅", "10日涨幅","20日涨幅","60日涨幅","120日涨幅","250日涨幅"]
    Fields_Query =  Fields_industry  + Fields_return_query
    #print(Fields_Query)
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_Query,query_type=QUERY_TYPE().stock)
    df.columns = ['股票代码', '股票简称','所属申万行业', '所属同花顺行业',"总市值",
                "5日涨幅", "10日涨幅","20日涨幅","60日涨幅","120日涨幅","250日涨幅",       
                "新股上市日期",'流通市值', '最新价', '最新涨跌幅','trade_date']
    df.columns = ['sec_code', 'sec_name','swl', 'thsl',"tmv"] + PERIODS_LIST().RT_LIST + ["list_date",'fmv', 'close', 'pct_chg','trade_date']
    for i in PERIODS().PERIODS:
        df[f"rs{i}"] = 100*df[f"rt{i}"].rank(axis=0,ascending=True, pct=True)
    data_to_excel(df, "stock_prs")
    return df


def stock_revenue_inc(years=3):
    """
    ['股票代码', '股票简称', '营业收入(同比增长率)', '营业收入(同比增长率)', '营业收入(同比增长率)',
       '营业收入(同比增长率)', '新股上市日期', '所属同花顺行业', '营业收入', '营业收入', '营业收入', '营业收入',
       '预测主营业务收入增长率', '最新价', '最新涨跌幅', 'trade_date']
    """
    Fields_query_inc = ["3年收入同比增长率"]
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_query_inc, query_type=QUERY_TYPE().stock)
    #print(df.columns)
    df.columns = ['sec_code', 'sec_name', 'revenue_inc_0', 'revunue_inc_L', 'revenue_inc_B','revenue_inc_BL',
    'list_date', 'thsl',  'revenue_inc_0','revunue_L', 'revenue_B','revenue_BL',
       'revenue_inc_forecast', 'close', 'pct_chg', 'trade_date']
    df = df[['sec_code', 'sec_name', 'revunue_inc_L', 'revenue_inc_B','revenue_inc_BL',
    'list_date', 'thsl', 'revunue_L', 'revenue_B','revenue_BL',
       'revenue_inc_forecast', 'close', 'pct_chg', 'trade_date']]
   
    data_to_excel(df, f"stock_revenue_inc_{years}years")
    return df


def stock_quarter_inc(quarters=4): #TODO
    """
    ['股票代码', '股票简称', '归属母公司股东的净利润(同比增长率)', '归属母公司股东的净利润(同比增长率)',
       '归属母公司股东的净利润(同比增长率)', '归属母公司股东的净利润(同比增长率)', '归属母公司股东的净利润(同比增长率)',
       '新股上市日期', '所属同花顺行业', '营业收入(同比增长率)', '营业收入(同比增长率)', '营业收入(同比增长率)',
       '营业收入(同比增长率)', '最新价', '最新涨跌幅', 'trade_date']
    
    """
    Fields_query_inc = ["4个季度的净利润同比增长率"]
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_query_inc, query_type=QUERY_TYPE().stock)
    print(df.columns)
   
    df.columns = ['sec_code', 'sec_name', 
                  'netProfit_Q_inc_0', 'netProfit_Q1_inc','netProfit_Q2_inc','netProfit_Q3_inc','netProfit_Q4_inc',
                'list_date', 'thsl',
                'revunue_Q1_inc', 'revenue_Q2_inc','revenue_Q3_inc','revenue_Q4_inc',
                'close', 'pct_chg', 'trade_date']

    df = df[['sec_code', 'netProfit_Q1_inc','netProfit_Q2_inc','netProfit_Q3_inc','netProfit_Q4_inc',
             'revunue_Q1_inc', 'revenue_Q2_inc','revenue_Q3_inc','revenue_Q4_inc']]
   
    data_to_excel(df, f"stock_income_inc_last{quarters}quarters")
    return df


def stock_year_inc(years=3):
    """
    ['股票代码', '股票简称', '归属母公司股东的净利润(同比增长率)', '归属母公司股东的净利润(同比增长率)',
       '归属母公司股东的净利润(同比增长率)', '归属母公司股东的净利润(同比增长率)', 
       '新股上市日期', '所属同花顺行业',
       'revenue_inc_0', 'revunue_inc_L', 'revenue_inc_B','revenue_inc_BL'
       #'营业收入(同比增长率)', '营业收入(同比增长率)', '营业收入(同比增长率)', '营业收入(同比增长率)',
       '销售毛利率', '最新价', '最新涨跌幅', 'trade_date']
    """
    Fields_query_inc = ["3年的利润同比增长率"]
    df = fetch_data_from_wencai(trade_date=None, fields_query=Fields_query_inc, query_type=QUERY_TYPE().stock)
    #print(df.columns)
    df.columns = ['sec_code', 'sec_name', 'netProfit_inc_0', 'netProfit_inc_L','netProfit_inc_B','netProfit_inc_BL',
       'list_date', 'thsl',
       'revenue_inc_0', 'revunue_inc_L', 'revenue_inc_B','revenue_inc_BL',
       'gpr', 'close', 'pct_chg', 'trade_date']
       
    df = df[['sec_code', 'sec_name', 'netProfit_inc_L','netProfit_inc_B','netProfit_inc_BL',
       'list_date', 'thsl',
        'revunue_inc_L', 'revenue_inc_B','revenue_inc_BL',
        'close', 'pct_chg', 'trade_date']]
    data_to_excel(df, f"stock_income_inc_last{years}years")
    return df


def stock_forecast():
    """ in: 
     预告净利润(all stock)
    out:
    ['股票代码', '股票简称', '预告净利润', '新股上市日期', '业绩预告类型', '上年同期净利润', '预告净利润变动幅度',
       '业绩预告日期', '业绩变化原因', '业绩预告摘要', '预告净利润上限', '预告净利润下限', 
       '所属同花顺行业', '最新价',   '最新涨跌幅', 'trade_date']
    in:业绩预告日期不为空(forecasted stock) 
    out:['股票代码', '股票简称', '业绩预告日期', '新股上市日期', '所属同花顺行业', '业绩预告类型', '预告净利润',
       '上年同期净利润', '预告净利润变动幅度', '业绩变化原因', '业绩预告摘要', '最新价', '最新涨跌幅',
       'trade_date']
    """
    Fields_Query = ["预告净利润"]
    df = fetch_data_from_wencai(trade_date=None,fields_query=Fields_Query,fields_out=None, query_type=QUERY_TYPE.stock)
    print(df.columns)
    df.columns = ['sec_code','sec_name','forecast_netProfit','list_date','forecast_type','last_year_netProfit','forecast_netProfit_pctchg',
     'date_forecast','change_reason','forcast_sammury',  'forecast_netProfit_uplimit', 'forecast_netProfit_downlimit',
            'thsl','close','pct_chg','trade_date']
    df.sort_values(by='date_forecast', ascending=False, inplace=True)
    data_to_excel(df, f"stock_forecast")
    return df


def stock_prs_oh_ol_volchg_inc(period=250, period_vol=50,period_history=0):
    df_prs = stock_prs()
    df_ma = stock_ma()[['sec_code','ma5', 'ma10', 'ma20', 'ma60', 'ma120', 'ma250']]
    df_vol = stock_vol_chg(period_vol=period_vol)[['sec_code',  'vol', f'vol_ma{period_vol}', f'vol_chg{period_vol}', 'amount', f'vol{period_vol}', f'turnover{period_vol}']]
    df_oh = stock_oh_ol(period=period)[['sec_code',  'H_0', 'OH_0', f'H_{period}',
       f'date_H_{period}', f'OH_{period}', f'L_{period}', f'date_L_{period}', f'OL_{period}',
       'open', 'high', 'low']]
    df_inc = stock_year_inc()[['sec_code','netProfit_inc_L','netProfit_inc_B','netProfit_inc_BL',
                                 'revunue_inc_L', 'revenue_inc_B','revenue_inc_BL']]
    df = pd.merge(df_prs, df_ma, on="sec_code")
    df = pd.merge(df, df_vol, on="sec_code")
    df = pd.merge(df, df_oh, on="sec_code")
    df = pd.merge(df, df_inc, on="sec_code")
    #print(df.columns)
    df = df[['trade_date','sec_code', 'sec_name', 'swl', 'thsl', 'tmv','fmv','rt5', 'rt10', 'rt20',
       'rt60', 'rt120', 'rt250', 'list_date',  'close', 'open', 'high',
       'low','pct_chg',
        'rs5', 'rs10', 'rs20', 'rs60', 'rs120', 'rs250',
        'ma5', 'ma10', 'ma20', 'ma60', 'ma120', 'ma250', 
        'vol', 'vol_ma50','vol_chg50', 'amount', 'vol50', 'turnover50', 
        'H_0', 'OH_0', 'H_250','date_H_250', 'OH_250', 'L_250', 'date_L_250', 'OL_250',
        'netProfit_inc_L','netProfit_inc_B','netProfit_inc_BL',
        'revunue_inc_L', 'revenue_inc_B','revenue_inc_BL']]
    data_to_excel(df, f'stock_prs_oh_volchg_inc_{get_real_trade_date()}')
    return df
 
 
def stock_prs_oh_ol_volchg_inc_concept(period=250, period_vol=50,period_history=0):
    df_prs = stock_prs()
    df_ma = stock_ma()[['sec_code','ma5', 'ma10', 'ma20', 'ma60', 'ma120', 'ma250']]
    df_vol = stock_vol_chg(period_vol=period_vol)[['sec_code',  'vol', f'vol_ma{period_vol}', f'vol_chg{period_vol}', 'amount', f'vol{period_vol}', f'turnover{period_vol}']]
    df_oh = stock_oh_ol(period=period)[['sec_code',  'H_0', 'OH_0', f'H_{period}',
       f'date_H_{period}', f'OH_{period}', f'L_{period}', f'date_L_{period}', f'OL_{period}',
       'open', 'high', 'low']]
    df_inc = stock_year_inc()[['sec_code','netProfit_inc_L','netProfit_inc_B','netProfit_inc_BL',
                                 'revunue_inc_L', 'revenue_inc_B','revenue_inc_BL']]
    df_concept = stock_industry_concept()[['sec_code','concept']]
    df = pd.merge(df_prs, df_ma, on="sec_code")
    df = pd.merge(df, df_vol, on="sec_code")
    df = pd.merge(df, df_oh, on="sec_code")
    df = pd.merge(df, df_inc, on="sec_code")
    df = pd.merge(df,df_concept, on="sec_code")
    #print(df.columns)
    df = df[['trade_date','sec_code', 'sec_name', 'swl', 'thsl', 'concept','tmv','fmv','rt5', 'rt10', 'rt20',
       'rt60', 'rt120', 'rt250', 'list_date',  'close', 'open', 'high',
       'low','pct_chg',
        'rs5', 'rs10', 'rs20', 'rs60', 'rs120', 'rs250',
        'ma5', 'ma10', 'ma20', 'ma60', 'ma120', 'ma250', 
        'vol', 'vol_ma50','vol_chg50', 'amount', 'vol50', 'turnover50', 
        'H_0', 'OH_0', 'H_250','date_H_250', 'OH_250', 'L_250', 'date_L_250', 'OL_250',
        'netProfit_inc_L','netProfit_inc_B','netProfit_inc_BL',
        'revunue_inc_L', 'revenue_inc_B','revenue_inc_BL']]
    data_to_excel(df, f'stock_prs_oh_volchg_inc_concept_{get_real_trade_date()}')
    return df


 
def stock_prs_oh_ol_volchg_inc_forecast_concept(period=250, period_vol=50,period_history=0):
    df_prs = stock_prs()
    df_ma = stock_ma()[['sec_code','ma5', 'ma10', 'ma20', 'ma60', 'ma120', 'ma250']]
    df_vol = stock_vol_chg(period_vol=period_vol)[['sec_code',  'vol', f'vol_ma{period_vol}', f'vol_chg{period_vol}', 'amount', f'vol{period_vol}', f'turnover{period_vol}']]
    df_oh = stock_oh_ol(period=period)[['sec_code',  'H_0', 'OH_0', f'H_{period}',
       f'date_H_{period}', f'OH_{period}', f'L_{period}', f'date_L_{period}', f'OL_{period}'
       ]]
    df_inc = stock_year_inc()[['sec_code','netProfit_inc_L','netProfit_inc_B','netProfit_inc_BL',
                                 'revunue_inc_L', 'revenue_inc_B','revenue_inc_BL']]
    df_concept = stock_industry_concept()[['sec_code','concept']]
    df_forecast = stock_forecast()[['sec_code','forecast_netProfit_pctchg']]
    df = pd.merge(df_prs, df_ma, on="sec_code")
    df = pd.merge(df, df_vol, on="sec_code")
    df = pd.merge(df, df_oh, on="sec_code")
    df = pd.merge(df, df_inc, on="sec_code")
    df = pd.merge(df,df_concept, on="sec_code")
    df = pd.merge(df, df_forecast, on='sec_code')
    #print(df.columns)
    df = df[['trade_date','sec_code', 'sec_name', 'swl', 'thsl', 'concept','tmv','fmv','rt5', 'rt10', 'rt20',
       'rt60', 'rt120', 'rt250', 'list_date',  'close', 'pct_chg',
        'rs5', 'rs10', 'rs20', 'rs60', 'rs120', 'rs250',
        'ma5', 'ma10', 'ma20', 'ma60', 'ma120', 'ma250', 
        'vol', 'vol_ma50','vol_chg50', 'amount', 'vol50', 'turnover50', 
        'H_0', 'OH_0', 'H_250','date_H_250', 'OH_250', 'L_250', 'date_L_250', 'OL_250',
        'netProfit_inc_L','netProfit_inc_B','netProfit_inc_BL',
        'revunue_inc_L', 'revenue_inc_B','revenue_inc_BL','forecast_netProfit_pctchg']]
    data_to_excel(df, f'stock_prs_oh_volchg_inc_forecast_concept_{get_real_trade_date()}')
    return df
    

if __name__ == "__main__":
    #print(stock_industry_concept())
    #print(stock_ma())
    #print(stock_oh_ol()) 
    #print(stock_vol_chg())
    
    #print(stock_prs())

    #print(stock_year_inc())
    #print(stock_quarter_inc())
    #print(stock_forecast())
    
    #print(stock_prs_oh_ol_volchg_inc())
    #print(stock_prs_oh_ol_volchg_inc_concept())
    print(stock_prs_oh_ol_volchg_inc_forecast_concept())
    
    


#配置类,应用启动的时候根据运行的环境进行初始化
import logging
from configparser import ConfigParser

cf = None
def init(filename):
    import os
    if not os.path.isfile(filename):
        raise Exception('file not exists:%s' % filename)
    logging.info("初始化配置文件:%s" % filename)
    global cf
    cf = ConfigParser()
    cf.read(filename)

def get(sec, prop):
    if cf:
        return cf.get(sec, prop)
    else:
        raise Exception('initialize first')


if __name__=="__main__":
    init("config_dev.cfg")
    print(cf.get('db','host'))